import numpy as np
from numpy.testing import assert_array_equal, \
    assert_allclose
import unittest


def k_means_classify(data, centers):
    sq_dist = ((data[:,None,:] - centers[None,:,:])**2.).sum(axis=-1)
    indices = sq_dist.argmin(-1)

    return indices


def k_means_learn(data, centers):
    
    k, dim = centers.shape
    new_centers = np.empty((k, dim))
    
    indices = k_means_classify(data, centers)
    for idx in xrange(k):
        clusters = data[indices==idx,:]
        new_centers[idx,:] = clusters.mean(axis=0)
    
    return new_centers


class TestKMeans(unittest.TestCase):
    
    def test_k_means_classify(self):
        # Given
        centers = np.array([[1.0, +1.0],
                            [1.0, -1.0]])
        data = np.random.randn(100, 2)
        expected = np.where(data[:,1]>0.0, 0, 1)
        
        # When
        indices = k_means_classify(data, centers)
        
        # Then
        assert_array_equal(expected, indices)
        
    def test_k_means_learn(self):
        # Given
        n_samples = 500
        dim = 2
        k = 2
        
        data = np.random.rand(n_samples, dim)
        data[0::2,0] -= 10.0
        data[1::2,0] += 10.0
        
        expected = np.array([[-9.5, 0.5],
                             [10.5, 0.5]])
                             
        centers = data[:2,:]
        
        # When
        for _ in range(100):
            centers = k_means_learn(data, centers)
        
        # Then
        assert_allclose(expected, centers, rtol=1e-1)
        
if __name__ == '__main__':
    unittest.main()
